@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2001-2002 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2001-2011 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -339,7 +339,7 @@ RM_STATUS NV_API_CALL os_queue_work_item(void *data)
     nvidia_work_t *work;
 
     status = os_alloc_mem((void **)&work, sizeof(nvidia_work_t));
-    if (status != status)
+    if (status != RM_OK)
         return status;
 
     work->data = data;
@@ -417,16 +417,13 @@ void NV_API_CALL os_log_error(
     NvU32 log_level,
     NvU32 error_number,
     const char *fmt,
-    void *ap
+    va_list ap
 )
 {
     int l;
     sprintf(nv_error_string, "NVRM: ");
     l = strlen(nv_error_string);
-    if (ap != NULL)
-        vsnprintf(nv_error_string + l, MAX_ERROR_STRING - l, fmt, ap);
-    else
-        snprintf(nv_error_string + l, MAX_ERROR_STRING - l, fmt);
+    vsnprintf(nv_error_string + l, MAX_ERROR_STRING - l, fmt, ap);
     printf("%s", nv_error_string);
 }
 
@@ -683,60 +680,58 @@ BOOL NV_API_CALL os_is_acquired_sema(void *semaphore)
 RM_STATUS NV_API_CALL os_alloc_mutex(void **mutex)
 {
     RM_STATUS status;
-    struct mtx *mtx;
+    struct sx *sx;
 
-    status = os_alloc_mem((void **)&mtx, sizeof(struct mtx));
+    status = os_alloc_mem((void **)&sx, sizeof(struct sx));
     if (status != RM_OK)
         return status;
 
-    mtx_init(mtx, "mtx", NULL, MTX_DEF | MTX_RECURSE);
-
-    *mutex = (void *)mtx;
+    sx_init(sx, "os.lock_sx");
+    *mutex = (void *)sx;
 
     return RM_OK;
 }
 
 void NV_API_CALL os_free_mutex(void *mutex)
 {
-    struct mtx *mtx = mutex;
+    struct sx *sx = mutex;
 
-    if (mtx != NULL) {
-        mtx_assert(mtx, MA_OWNED);
-        mtx_destroy(mtx);
-        os_free_mem(mtx);
+    if (sx != NULL) {
+        sx_destroy(sx);
+        os_free_mem(sx);
     }
 }
 
 RM_STATUS NV_API_CALL os_acquire_mutex(void *mutex)
 {
-    struct mtx *mtx = mutex;
+    struct sx *sx = mutex;
 
-    if (__NV_ITHREAD()) {
+    if (__NV_ITHREAD())
         return RM_ERR_INVALID_REQUEST;
-    }
 
-    mtx_lock(mtx);
+    sx_xlock(sx);
+
     return RM_OK;
 }
 
 RM_STATUS NV_API_CALL os_cond_acquire_mutex(void *mutex)
 {
-    struct mtx *mtx = mutex;
-    if (__NV_ITHREAD()) {
-        return RM_ERR_INVALID_REQUEST;
-    }
+    struct sx *sx = mutex;
 
-    if (mtx_trylock(mtx) == 0) {
+    if (__NV_ITHREAD())
+        return RM_ERR_INVALID_REQUEST;
+
+    if (sx_try_xlock(sx) == 0)
         return RM_ERR_TIMEOUT_RETRY;
-    }
 
     return RM_OK;
 }
 
 void NV_API_CALL os_release_mutex(void *mutex)
 {
-    struct mtx *mtx = mutex;
-    mtx_unlock(mtx);
+    struct sx *sx = mutex;
+
+    sx_xunlock(sx);
 }
 
 struct os_semaphore {
@@ -746,7 +741,6 @@ struct os_semaphore {
     NvS32  limit;
 };
 
-
 void* NV_API_CALL os_alloc_semaphore(NvU32 initialValue, NvU32 limit)
 {
     RM_STATUS status;
@@ -758,10 +752,10 @@ void* NV_API_CALL os_alloc_semaphore(NvU32 initialValue, NvU32 limit)
         return NULL;
     }
 
-    mtx_init(&os_sema->mutex_mtx, "os.sema_mtx", NULL, MTX_DEF | MTX_RECURSE);
+    mtx_init(&os_sema->mutex_mtx, "os.sema_mtx", NULL, MTX_DEF);
     cv_init(&os_sema->mutex_cv, "os.sema_cv");
 
-    os_sema->count = 1;
+    os_sema->count = initialValue;
     os_sema->limit = limit;
 
     return (void *)os_sema;
@@ -783,10 +777,10 @@ RM_STATUS NV_API_CALL os_acquire_semaphore(void *semaphore)
 
     mtx_lock(&os_sema->mutex_mtx);
     os_sema->count--;
-    if (os_sema->count < 0) {
-        cv_wait(&os_sema->mutex_cv, &os_sema->mutex_mtx);
-    }
-    mtx_unlock(&os_sema->mutex_mtx);
+    if (os_sema->count < 0)
+        cv_wait_unlock(&os_sema->mutex_cv, &os_sema->mutex_mtx);
+    else
+        mtx_unlock(&os_sema->mutex_mtx);
 
     return RM_OK;
 }
@@ -810,6 +804,11 @@ BOOL NV_API_CALL os_semaphore_may_sleep(void)
     return (!__NV_ITHREAD());
 }
 
+BOOL NV_API_CALL os_is_isr(void)
+{
+    return (__NV_ITHREAD());
+}
+
 BOOL NV_API_CALL os_pat_supported(void)
 {
     return TRUE;
@@ -839,20 +838,47 @@ RM_STATUS NV_API_CALL os_enable_console_access(void)
     return RM_OK;
 }
 
-NvU64 NV_API_CALL os_acquire_spinlock(void *pSema)
+RM_STATUS NV_API_CALL os_alloc_spinlock(void **lock)
 {
-    struct os_mutex *mtx = pSema;
+    RM_STATUS status;
+    struct mtx *mtx;
 
-    mtx_lock(&mtx->mutex_mtx);
+    status = os_alloc_mem((void **)&mtx, sizeof(struct mtx));
+    if (status != RM_OK)
+        return status;
+
+    mtx_init(mtx, "os.lock_mtx", NULL, MTX_DEF);
+
+    *lock = (void *)mtx;
+
+    return RM_OK;
+}
+
+void NV_API_CALL os_free_spinlock(void *lock)
+{
+    struct mtx *mtx = lock;
+
+    if (mtx != NULL) {
+        mtx_assert(mtx, MA_OWNED);
+        mtx_destroy(mtx);
+        os_free_mem(mtx);
+    }
+}
+
+NvU64 NV_API_CALL os_acquire_spinlock(void *lock)
+{
+    struct mtx *mtx = lock;
+
+    mtx_lock(mtx);
 
     return 0;
 }
 
-void NV_API_CALL os_release_spinlock(void *pSema, NvU64 oldIrql)
+void NV_API_CALL os_release_spinlock(void *lock, NvU64 oldIrql)
 {
-    struct os_mutex *mtx = pSema;
+    struct mtx *mtx = lock;
 
-    mtx_unlock(&mtx->mutex_mtx);
+    mtx_unlock(mtx);
 }
 
 RM_STATUS NV_API_CALL os_get_address_space_info(
